import requests
import pandas as pd


def get_public_ip():
    resp = requests.get("https://www.whatismyip.org/my-ip-address")
    df = pd.read_html(resp.text)[0]
    df.columns = ["labels", "vals"]
    return df[df.labels == "Your IP"].vals[0]



def retry(f, n, *fargs, **fkwargs):
    for t in range(n):
        try:
            res = f(*fargs, **fkwargs)
            return res
        except ValueError:
            print(f"Failed {t} time(s); trying again {n-t} more time(s).")
            continue



from typing import Iterable
import ray


@ray.remote
def _http_remote(f, payload: dict):
    resp = f(**payload)
    return resp


def http_parallel(f, payloads: Iterable[dict]):
    if ray.is_initialized():
        ray.shutdown()
    ray.init()
    results = ray.get([_http_remote.remote(f, payload) for payload in payloads])
    ray.shutdown()
    return results
